#include <cstdint>
#include "testlib.h"

using namespace std;

vector<int> make_partition (int m, int k) {
  vector<int> bars;
  bars.push_back(0);
  for (int i = 0; i < k - 1; i++)
    bars.push_back(rnd.next(0, m));
  bars.push_back(m);

  sort(bars.begin(), bars.end());
  
  vector<int> ans (k);
  for (int i = 1; i <= k; i++)
    ans[i - 1] = bars[i] - bars[i - 1];

  return ans;
}

void build_intervals (int u, const vector<vector<int>> &tree, const vector<int> &partition,
                      int &ptr, int &pos, vector<pair<int, int>> &intervals) {
  int l = pos;

  for (int i = 0; i < (int) tree[u].size(); i++) {
    pos += partition[ptr];
    ptr++;

    build_intervals(tree[u][i], tree, partition, ptr, pos, intervals);
  }

  pos += partition[ptr];
  ptr++;

  if (tree[u].empty())
    pos++;

  int r = pos;
  if (u != 0)
    intervals.emplace_back(l + 1, r);
}

int main (int argc, char **argv) {
  registerGen(argc, argv, 1);

  int n = opt<int>("n");
  int m = opt<int>("m");
  int wnx = opt<int>("wnx");

  ensuref(n <= m, "n <= m must hold to use the tree generator");

  // first, generate a tree of 0...n
  // 0 does not correspond to a segment - it is instead the whole segment
  vector<vector<int>> ch (n + 1, vector<int> (0));
  for (int i = 1; i <= n; i++) {
    int p = rnd.wnext(0, i - 1, wnx);
    ch[p].push_back(i);
  }

  // ...then, allocate excess space to each vertex
  int excess = m, gaps = 0;
  for (int i = 0; i <= n; i++) {
    if (ch[i].size() == 0) {
      excess--;
    }
    gaps += ch[i].size() + 1;
  }

  auto partition = make_partition(excess, gaps);
  vector<pair<int, int>> intervals;

  int ptr = 0, pos = 0;
  build_intervals(0, ch, partition, ptr, pos, intervals);

  shuffle(intervals.begin(), intervals.end());

  cout << n << " " << m << '\n';
  for (int i = 0; i < n; i++)
    cout << intervals[i].first << " " << intervals[i].second << '\n';
}
