#include <cstdint>
#include "testlib.h"

using namespace std;

struct Event {
  int time, type, other, id;
};

bool operator< (Event p, Event q) {
  if (p.time != q.time)
    return p.time < q.time;

  // opening before closing
  if (p.type != q.type)
    return p.type < q.type;

  // always bigger other first
  if (p.other != q.other)
    return p.other > q.other;

  // break ties by id
  return p.type ^ (p.id < q.id);
}

bool is_tree (const vector<pair<int, int>> &intervals) {
  vector<Event> events;
  for (int i = 0; i < (int) intervals.size(); i++) {
    auto I = intervals[i];
    events.push_back(Event {.time = I.first, .type = 0, .other = I.second, .id = i});
    events.push_back(Event {.time = I.second, .type = 1, .other = I.first, .id = i});
  }

  sort(events.begin(), events.end());

  vector<int> stk;
  for (auto e : events) {
    if (e.type == 0) {
      stk.push_back(e.id);
    } else {
      if (stk.empty() || stk.back() != e.id)
        return false;
      stk.pop_back();
    }
  }

  return true;
}

int main (int argc, char **argv) {
  registerValidation(argc, argv);

  int n = inf.readInt(1, 200'000, "n");
  inf.readSpace();
  int m = inf.readInt(1, 1'000'000'000, "m");
  inf.readEoln();

  vector<pair<int, int>> intervals;
  for (int i = 0; i < n; i++) {
    int l = inf.readInt(1, m, "li");
    inf.readSpace();
    int r = inf.readInt(l, m, "ri");
    inf.readEoln();

    intervals.emplace_back(l, r);
  }

  inf.readEof();

  if (validator.group() == "n-and-m-small") {
    ensure(n <= 100);
    ensure(m <= 100);
  }

  if (validator.group() == "n-<=-1000")
    ensure(n <= 1000);

  if (validator.group() == "m-<=-1000")
    ensure(m <= 1000);

  if (validator.group() == "m-<=-1e5")
    ensure(m <= 100'000);

  if (validator.group() == "treelike")
    ensure(is_tree(intervals));
}
